@@ -134,7 +134,7 @@ main (int argc, char **argv)
     mode |= usrp_standard_rx::FPGA_MODE_COUNTING;
 
 
-  usrp_standard_rx *urx = usrp_standard_rx::make (which_board, decim, mode);
+  usrp_standard_rx *urx = usrp_standard_rx::make (which_board, decim, 1, -1, mode);
   if (urx == 0)
     die ("usrp_standard_rx::make");
 
