@@ -27,7 +27,7 @@
 #include "usrp_basic.h"
 #include "usrp_prims.h"
 #include "usrp_interfaces.h"
-#include "fpga_regs.h"
+#include "fpga_regs_common.h"
 #include "fusb.h"
 #include <usb.h>
 #include <stdexcept>
@@ -121,8 +121,8 @@ usrp_basic::usrp_basic (int which_board,
     throw std::runtime_error ("usrp_basic/usrp_find_device");
   }
 
-  if (!usrp_usrp1_p (dev)){
-    fprintf (stderr, "usrp_basic: sorry, this code only works with Rev 1 USRPs\n");
+  if (!(usrp_usrp1_p (dev) || usrp_usrp2_p (dev))){
+    fprintf (stderr, "usrp_basic: sorry, this code only works with Rev 1 and 2 USRPs\n");
     throw std::runtime_error ("usrp_basic/bad_rev");
   }
 
@@ -205,8 +205,10 @@ usrp_basic::read_eeprom (int i2c_addr, int eeprom_offset, int len)
 bool
 usrp_basic::_write_fpga_reg (int regno, int value)
 {
-  if (d_verbose)
-    printf ("_write_fpga_reg (%3d, 0x%08x)\n", regno, value);
+  if (d_verbose){
+    fprintf (stdout, "_write_fpga_reg (%3d, 0x%08x)\n", regno, value);
+    fflush (stdout);
+  }
   return usrp_write_fpga_reg (d_udh, regno, value);
 }
 
@@ -233,13 +235,13 @@ usrp_basic::_write_9862 (int which_codec, int regno, unsigned char value)
 
 
 bool
-usrp_basic::_read_9862 (int which_codec, int regno, unsigned char *value)
+usrp_basic::_read_9862 (int which_codec, int regno, unsigned char *value) const
 {
   return usrp_9862_read (d_udh, which_codec, regno, value);
 }
 
 int
-usrp_basic::_read_9862 (int which_codec, int regno)
+usrp_basic::_read_9862 (int which_codec, int regno) const
 {
   unsigned char value;
   if (!_read_9862 (which_codec, regno, &value))
@@ -271,7 +273,7 @@ static unsigned char rx_init_regs[] = {
   REG_RX_PWR_DN,	0,
   REG_RX_A,		0,	// minimum gain = 0x00 (max gain = 0x14)
   REG_RX_B,		0,	// minimum gain = 0x00 (max gain = 0x14)
-  REG_RX_MISC,		0,	// consider (RX_MISC_HS_DUTY_CYCLE | RX_MISC_HS_CLK_DUTY)
+  REG_RX_MISC,		(RX_MISC_HS_DUTY_CYCLE | RX_MISC_CLK_DUTY),
   REG_RX_IF,		(RX_IF_USE_CLKOUT1
 			 | RX_IF_2S_COMP),
   REG_RX_DIGITAL,	(RX_DIGITAL_2_CHAN)
@@ -291,8 +293,10 @@ usrp_basic_rx::usrp_basic_rx (int which_board)
     throw std::runtime_error ("usrp_basic_rx/init_9862");
   }
 
-  // FIXME power down 2nd codec rx path
-  usrp_9862_write (d_udh, 1, REG_RX_PWR_DN, 0x1);	// power down everything
+  if (0){
+    // FIXME power down 2nd codec rx path
+    usrp_9862_write (d_udh, 1, REG_RX_PWR_DN, 0x1);	// power down everything
+  }
 
   // Reset the rx path and leave it disabled.
   set_rx_enable (false);
@@ -301,6 +305,8 @@ usrp_basic_rx::usrp_basic_rx (int which_board)
 
   set_fpga_rx_sample_rate_divisor (2);	// usually correct
 
+  probe_rx_slots (true);
+
   d_devhandle = fusb_sysconfig::make_devhandle (d_udh);
   d_ephandle = d_devhandle->make_ephandle (USRP_RX_ENDPOINT, true,
 					   FUSB_BLOCK_SIZE, FUSB_NBLOCKS);
@@ -433,6 +439,163 @@ usrp_basic_rx::set_rx_enable (bool on)
   return usrp_set_fpga_rx_enable (d_udh, on);
 }
 
+// conditional disable, return prev state
+bool
+usrp_basic_rx::disable_rx ()
+{
+  bool enabled = rx_enable ();
+  if (enabled)
+    set_rx_enable (false);
+  return enabled;
+}
+
+// conditional set
+void
+usrp_basic_rx::restore_rx (bool on)
+{
+  if (on != rx_enable ())
+    set_rx_enable (on);
+}
+
+bool
+usrp_basic_rx::set_pga (int which, double gain)
+{
+  if (which < 0 || which > 3)
+    return false;
+
+  gain = std::max (pga_min (), gain);
+  gain = std::min (pga_max (), gain);
+
+  int codec = which >> 1;
+  int reg = (which & 1) == 0 ? REG_RX_A : REG_RX_B;
+
+  int int_gain = (int) rint ((gain - pga_min ()) / pga_db_per_step());
+
+  return _write_9862 (codec, reg, int_gain);
+}
+
+double
+usrp_basic_rx::pga (int which) const
+{
+  if (which < 0 || which > 3)
+    return READ_FAILED;
+
+  int codec = which >> 1;
+  int reg = (which & 1) == 0 ? REG_RX_A : REG_RX_B;
+  unsigned char v;
+  bool ok = _read_9862 (codec, reg, &v);
+  if (!ok)
+    return READ_FAILED;
+
+  return (pga_db_per_step() * (v & 0x1f)) + pga_min();
+}
+
+static int
+slot_id_to_oe_reg (int slot_id)
+{
+  static int reg[4]  = { FR_OE_0, FR_OE_1, FR_OE_2, FR_OE_3 };
+  assert (0 <= slot_id && slot_id < 4);
+  return reg[slot_id];
+}
+
+static int
+slot_id_to_io_reg (int slot_id)
+{
+  static int reg[4]  = { FR_IO_0, FR_IO_1, FR_IO_2, FR_IO_3 };
+  assert (0 <= slot_id && slot_id < 4);
+  return reg[slot_id];
+}
+
+void
+usrp_basic_rx::probe_rx_slots (bool verbose)
+{
+  struct usrp_dboard_eeprom	eeprom;
+  static int slot_id_map[2] = { SLOT_RX_A, SLOT_RX_B };
+  static const char *slot_name[2] = { "RX d'board A", "RX d'board B" };
+
+  for (int i = 0; i < 2; i++){
+    int slot_id = slot_id_map [i];
+    const char *msg = 0;
+    unsigned offset;
+    usrp_dbeeprom_status_t s = usrp_read_dboard_eeprom (d_udh, slot_id, &eeprom);
+
+    switch (s){
+    case UDBE_OK:
+      d_dbid[i] = eeprom.id;
+      msg = usrp_dbid_to_string (eeprom.id).c_str ();
+      offset = (eeprom.offset[1] << 16) | (eeprom.offset[0] & 0xffff);
+      _write_fpga_reg (i == 0 ? FR_ADC_OFFSET_1_0 : FR_ADC_OFFSET_3_2, offset);
+      _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | eeprom.oe);
+      break;
+      
+    case UDBE_NO_EEPROM:
+      d_dbid[i] = -1;
+      msg = "<none>";
+      _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      break;
+      
+    case UDBE_INVALID_EEPROM:
+      d_dbid[i] = -2;
+      msg = "Invalid EEPROM contents";
+      _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      break;
+      
+    case UDBE_BAD_SLOT:
+    default:
+      assert (0);
+    }
+
+    if (verbose){
+      fflush (stdout);
+      fprintf (stderr, "%s: %s\n", slot_name[i], msg);
+    }
+  }
+}
+
+bool
+usrp_basic_rx::_write_oe (int which_dboard, int value, int mask)
+{
+  if (! (0 <= which_dboard && which_dboard <= 1))
+    return false;
+
+  return _write_fpga_reg (slot_id_to_oe_reg (dboard_to_slot (which_dboard)),
+			  (mask << 16) | (value & 0xffff));
+}
+
+bool
+usrp_basic_rx::write_io (int which_dboard, int value, int mask)
+{
+  if (! (0 <= which_dboard && which_dboard <= 1))
+    return false;
+
+  return _write_fpga_reg (slot_id_to_io_reg (dboard_to_slot (which_dboard)),
+			  (mask << 16) | (value & 0xffff));
+}
+
+bool
+usrp_basic_rx::read_io (int which_dboard, int *value)
+{
+  if (! (0 <= which_dboard && which_dboard <= 1))
+    return false;
+
+  int t;
+  int reg = which_dboard + 1;	// FIXME, *very* magic number (fix in serial_io.v)
+  bool ok = _read_fpga_reg (reg, &t);
+  if (!ok)
+    return false;
+
+  *value = (t >> 16) & 0xffff;	// FIXME, more magic
+  return true;
+}
+
+int
+usrp_basic_rx::read_io (int which_dboard)
+{
+  int	value;
+  if (!read_io (which_dboard, &value))
+    return READ_FAILED;
+  return value;
+}
 
 ////////////////////////////////////////////////////////////////
 //
@@ -484,10 +647,12 @@ usrp_basic_tx::usrp_basic_tx (int which_board)
     throw std::runtime_error ("usrp_basic_tx/init_9862");
   }
 
-  // FIXME power down 2nd codec tx path
-  usrp_9862_write (d_udh, 1, REG_TX_PWR_DN,
-		   (TX_PWR_DN_TX_DIGITAL
-		    | TX_PWR_DN_TX_ANALOG_BOTH));
+  if (0){
+    // FIXME power down 2nd codec tx path
+    usrp_9862_write (d_udh, 1, REG_TX_PWR_DN,
+		     (TX_PWR_DN_TX_DIGITAL
+		      | TX_PWR_DN_TX_ANALOG_BOTH));
+  }
 
   // Reset the tx path and leave it disabled.
   set_tx_enable (false);
@@ -496,6 +661,8 @@ usrp_basic_tx::usrp_basic_tx (int which_board)
 
   set_fpga_tx_sample_rate_divisor (4);	// we're using interp x4
 
+  probe_tx_slots (true);
+
   d_devhandle = fusb_sysconfig::make_devhandle (d_udh);
   d_ephandle = d_devhandle->make_ephandle (USRP_TX_ENDPOINT, false,
 					   FUSB_BLOCK_SIZE, FUSB_NBLOCKS);
@@ -630,3 +797,141 @@ usrp_basic_tx::set_tx_enable (bool on)
   return usrp_set_fpga_tx_enable (d_udh, on);
 }
 
+// conditional disable, return prev state
+bool
+usrp_basic_tx::disable_tx ()
+{
+  bool enabled = tx_enable ();
+  if (enabled)
+    set_tx_enable (false);
+  return enabled;
+}
+
+// conditional set
+void
+usrp_basic_tx::restore_tx (bool on)
+{
+  if (on != tx_enable ())
+    set_tx_enable (on);
+}
+
+bool
+usrp_basic_tx::set_pga (int which, double gain)
+{
+  if (which < 0 || which > 3)
+    return false;
+
+  gain = std::max (pga_min (), gain);
+  gain = std::min (pga_max (), gain);
+
+  int codec = which >> 1;	// 0 and 1 are same, as are 2 and 3
+
+  int int_gain = (int) rint ((gain - pga_min ()) / pga_db_per_step());
+
+  return _write_9862 (codec, REG_TX_PGA, int_gain);
+}
+
+double
+usrp_basic_tx::pga (int which) const
+{
+  if (which < 0 || which > 3)
+    return READ_FAILED;
+
+  int codec = which >> 1;
+  unsigned char v;
+  bool ok = _read_9862 (codec, REG_TX_PGA, &v);
+  if (!ok)
+    return READ_FAILED;
+
+  return (pga_db_per_step() * v) + pga_min();
+}
+
+void
+usrp_basic_tx::probe_tx_slots (bool verbose)
+{
+  struct usrp_dboard_eeprom	eeprom;
+  static int slot_id_map[2] = { SLOT_TX_A, SLOT_TX_B };
+  static const char *slot_name[2] = { "TX d'board A", "TX d'board B" };
+
+  for (int i = 0; i < 2; i++){
+    int slot_id = slot_id_map [i];
+    const char *msg = 0;
+    usrp_dbeeprom_status_t s = usrp_read_dboard_eeprom (d_udh, slot_id, &eeprom);
+
+    switch (s){
+    case UDBE_OK:
+      d_dbid[i] = eeprom.id;
+      msg = usrp_dbid_to_string (eeprom.id).c_str ();
+      // FIXME, figure out interpretation of dc offset for TX d'boards
+      // offset = (eeprom.offset[1] << 16) | (eeprom.offset[0] & 0xffff);
+      _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | eeprom.oe);
+      break;
+      
+    case UDBE_NO_EEPROM:
+      d_dbid[i] = -1;
+      msg = "<none>";
+      _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      break;
+      
+    case UDBE_INVALID_EEPROM:
+      d_dbid[i] = -2;
+      msg = "Invalid EEPROM contents";
+      _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      break;
+      
+    case UDBE_BAD_SLOT:
+    default:
+      assert (0);
+    }
+
+    if (verbose){
+      fflush (stdout);
+      fprintf (stderr, "%s: %s\n", slot_name[i], msg);
+    }
+  }
+}
+
+bool
+usrp_basic_tx::_write_oe (int which_dboard, int value, int mask)
+{
+  if (! (0 <= which_dboard && which_dboard <= 1))
+    return false;
+
+  return _write_fpga_reg (slot_id_to_oe_reg (dboard_to_slot (which_dboard)),
+			  (mask << 16) | (value & 0xffff));
+}
+
+bool
+usrp_basic_tx::write_io (int which_dboard, int value, int mask)
+{
+  if (! (0 <= which_dboard && which_dboard <= 1))
+    return false;
+
+  return _write_fpga_reg (slot_id_to_io_reg (dboard_to_slot (which_dboard)),
+			  (mask << 16) | (value & 0xffff));
+}
+
+bool
+usrp_basic_tx::read_io (int which_dboard, int *value)
+{
+  if (! (0 <= which_dboard && which_dboard <= 1))
+    return false;
+
+  int t;
+  int reg = which_dboard + 1;	// FIXME, *very* magic number (fix in serial_io.v)
+  bool ok = _read_fpga_reg (reg, &t);
+  if (!ok)
+    return false;
+
+  *value = t & 0xffff;		// FIXME, more magic
+  return true;
+}
+
+int
+usrp_basic_tx::read_io (int which_dboard)
+{
+  int	value;
+  if (!read_io (which_dboard, &value))
+    return READ_FAILED;
+  return value;
+}
