@@ -20,9 +20,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "usrp_prims.h"
 #include "usrp_commands.h"
 #include "usrp_ids.h"
+#include "usrp_i2c_addr.h"
 #include <usb.h>
 #include <errno.h>
 #include <stdio.h>
@@ -34,6 +39,8 @@
 #include <algorithm>
 #include <ad9862.h>
 
+#define VERBOSE 0
+
 using namespace ad9862;
 
 static const int FIRMWARE_HASH_SLOT	= 0;
@@ -154,6 +161,12 @@ usrp_usrp1_p (struct usb_device *q)
   return usrp_usrp_p (q) && _usrp_hw_rev (q) == 1;
 }
 
+bool
+usrp_usrp2_p (struct usb_device *q)
+{
+  return usrp_usrp_p (q) && _usrp_hw_rev (q) == 2;
+}
+
 
 bool
 usrp_unconfigured_usrp_p (struct usb_device *q)
@@ -178,7 +191,7 @@ usrp_find_device (int nth, bool fx2_ok_p)
 
   usrp_one_time_init ();
   
-  p = usb_busses;
+  p = usb_get_busses();
   while (p != NULL){
     q = p->devices;
     while (q != NULL){
@@ -206,12 +219,23 @@ usrp_open_interface (struct usb_device *dev, int interface, int altinterface)
     abort ();
   }
 
+  if (usb_set_configuration (udh, 1) < 0){
+    fprintf (stderr, "%s: usb_claim_interface: failed conf %d\n", __FUNCTION__,interface);
+    fprintf (stderr, "%s\n", usb_strerror());
+    usb_close (udh);
+    return 0;
+  }
+
   if (usb_claim_interface (udh, interface) < 0){
+    fprintf (stderr, "%s:usb_claim_interface: failed interface %d\n", __FUNCTION__,interface);
+    fprintf (stderr, "%s\n", usb_strerror());
     usb_close (udh);
     return 0;
   }
 
   if (usb_set_altinterface (udh, altinterface) < 0){
+    fprintf (stderr, "%s:usb_set_alt_interface: failed\n", __FUNCTION__);
+    fprintf (stderr, "%s\n", usb_strerror());
     usb_release_interface (udh, interface);
     usb_close (udh);
     return 0;
@@ -567,6 +591,7 @@ usrp_write_fpga_reg (struct usb_dev_handle *udh, int reg, int value)
   case 0:
     return usrp0_fpga_write (udh, reg, value);
   case 1:
+  case 2:
     return usrp1_fpga_write (udh, reg, value);
   default:
     abort ();
@@ -580,6 +605,7 @@ usrp_read_fpga_reg (struct usb_dev_handle *udh, int reg, int *value)
   case 0:
     return false;		// no read back
   case 1:
+  case 2:
     return usrp1_fpga_read (udh, reg, value);
   default:
     abort ();
@@ -840,6 +866,7 @@ load_status_msg (usrp_load_status_t s, const char *type, const char *filename)
     fprintf (stderr, "usrp: failed to load %s %s.\n", type, filename);
     break;
     
+#if (VERBOSE)
   case ULS_ALREADY_LOADED:
     fprintf (stderr, "usrp: %s %s already loaded.\n", type, filename);
     break;
@@ -847,6 +874,11 @@ load_status_msg (usrp_load_status_t s, const char *type, const char *filename)
   case ULS_OK:
     fprintf (stderr, "usrp: %s %s loaded successfully.\n", type, filename);
     break;
+#else
+  case ULS_ALREADY_LOADED:
+  case ULS_OK:
+    break;
+#endif
   }
 }
 
@@ -1058,6 +1090,7 @@ power_down_9862s (struct usb_dev_handle *udh)
 
   switch (_usrp_hw_rev (dev_handle_to_dev (udh))){
   case 1:
+  case 2:
     usrp_9862_write_many_all (udh, regs, sizeof (regs));
     break;
   }
@@ -1240,3 +1273,90 @@ usrp_read_aux_adc (struct usb_dev_handle *udh, int slot,
   
   return r;
 }
+
+// ----------------------------------------------------------------
+
+static int slot_to_i2c_addr (int slot)
+{
+  switch (slot){
+  case SLOT_TX_A:	return I2C_ADDR_TX_A;
+  case SLOT_RX_A:	return I2C_ADDR_RX_A;
+  case SLOT_TX_B:	return I2C_ADDR_TX_B;
+  case SLOT_RX_B:	return I2C_ADDR_RX_B;
+  default:		return -1;
+  }
+}
+
+static void
+set_chksum (unsigned char *buf)
+{
+  int sum = 0;
+  unsigned int i;
+  for (i = 0; i < DB_EEPROM_CLEN - 1; i++)
+    sum += buf[i];
+  buf[i] = -sum;
+}
+
+static usrp_dbeeprom_status_t
+read_dboard_eeprom (struct usb_dev_handle *udh,
+		    int slot_id, unsigned char *buf)
+{
+  int i2c_addr = slot_to_i2c_addr (slot_id);
+  if (i2c_addr == -1)
+    return UDBE_BAD_SLOT;
+
+  if (!usrp_eeprom_read (udh, i2c_addr, 0, buf, DB_EEPROM_CLEN))
+    return UDBE_NO_EEPROM;
+
+  if (buf[DB_EEPROM_MAGIC] != DB_EEPROM_MAGIC_VALUE)
+    return UDBE_INVALID_EEPROM;
+
+  int sum = 0;
+  for (unsigned int i = 0; i < DB_EEPROM_CLEN; i++)
+    sum += buf[i];
+
+  if ((sum & 0xff) != 0)
+    return UDBE_INVALID_EEPROM;
+
+  return UDBE_OK;
+}
+
+usrp_dbeeprom_status_t
+usrp_read_dboard_eeprom (struct usb_dev_handle *udh,
+			 int slot_id, usrp_dboard_eeprom *eeprom)
+{
+  unsigned char buf[DB_EEPROM_CLEN];
+
+  memset (eeprom, 0, sizeof (*eeprom));
+
+  usrp_dbeeprom_status_t s = read_dboard_eeprom (udh, slot_id, buf);
+  if (s != UDBE_OK)
+    return s;
+
+  eeprom->id = (buf[DB_EEPROM_ID_MSB] << 8) | buf[DB_EEPROM_ID_LSB];
+  eeprom->oe = (buf[DB_EEPROM_OE_MSB] << 8) | buf[DB_EEPROM_OE_LSB];
+  eeprom->offset[0] = (buf[DB_EEPROM_OFFSET_0_MSB] << 8) | buf[DB_EEPROM_OFFSET_0_LSB];
+  eeprom->offset[1] = (buf[DB_EEPROM_OFFSET_1_MSB] << 8) | buf[DB_EEPROM_OFFSET_1_LSB];
+
+  return UDBE_OK;
+}
+
+bool
+usrp_write_dboard_offsets (struct usb_dev_handle *udh, int slot_id,
+			   short offset0, short offset1)
+{
+  unsigned char buf[DB_EEPROM_CLEN];
+
+  usrp_dbeeprom_status_t s = read_dboard_eeprom (udh, slot_id, buf);
+  if (s != UDBE_OK)
+    return false;
+
+  buf[DB_EEPROM_OFFSET_0_LSB] = (offset0 >> 0) & 0xff;
+  buf[DB_EEPROM_OFFSET_0_MSB] = (offset0 >> 8) & 0xff;
+  buf[DB_EEPROM_OFFSET_1_LSB] = (offset1 >> 0) & 0xff;
+  buf[DB_EEPROM_OFFSET_1_MSB] = (offset1 >> 8) & 0xff;
+  set_chksum (buf);
+
+  return usrp_eeprom_write (udh, slot_to_i2c_addr (slot_id),
+			    0, buf, sizeof (buf));
+}
